# oops-repository is Copyright 2011 Canonical Ltd.
#
# Canonical Ltd ("Canonical") distributes the oops-repository source code under
# the GNU Affero General Public License, version 3 ("AGPLv3"). See the file
# LICENSE in the source tree for more information.

"""basic operations on oopses in the db."""

import json
import time
import uuid

import pycassa
from pycassa.index import create_index_expression, LT, create_index_clause
from pycassa.cassandra.ttypes import IndexExpression

DAY = 60*60*24
MONTH = DAY*30

def prune(config):
    """Remove OOPSES that are over 30 days old."""
    pool = pycassa.connect(config['keyspace'])
    dayoops_cf = pycassa.ColumnFamily(pool, 'DayOOPS')
    oops_cf = pycassa.ColumnFamily(pool, 'OOPS')
    # Find days to prune
    days = set()
    prune_to = time.strftime('%Y%m%d', time.gmtime(time.time() - MONTH))
    for key, _ in dayoops_cf.get_range(columns=()):
        if key < prune_to:
            days.add(key)
    if not days:
        return
    # collect all the oopses (buffers all in memory; may want to make
    # incremental in future)
    batch_size = 10000
    oopses = []
    for day in days:
        columns_found = batch_size
        start_col = ''
        while columns_found==batch_size:
            columns = dayoops_cf.get(day, column_start=start_col)
            columns_found = len(columns)
            for column, oopsid in columns.items():
                start_col = column
                oopses.append(oopsid)
    # Remove the oopses
    batch = oops_cf.batch()
    map(batch.remove, oopses)
    batch.send()
    # Clean out the days aggregates
    # Clean out the days index
    batch = dayoops_cf.batch()
    map(batch.remove, days)
    batch.send()


def insert(config, oopsid, oops_json):
    """Insert an OOPS into the system.
    
    :return: The day which the oops was filed under.
    """
    pool = pycassa.connect(config['keyspace'])
    dayoops_cf = pycassa.ColumnFamily(pool, 'DayOOPS')
    oops_cf = pycassa.ColumnFamily(pool, 'OOPS')
    day_key = time.strftime('%Y%m%d', time.gmtime())
    now_uuid = uuid.uuid1()
    # make sure the oops report is a json dict, and break out each key to a
    # separate column. For now, rather than worrying about typed column values
    # we just coerce them all to strings.
    oops_dict = json.loads(oops_json)
    assert isinstance(oops_dict, dict)
    insert_dict = {}
    for key, value in oops_dict.items():
        insert_dict[key] = json.dumps(value)
    oops_cf.insert(oopsid, insert_dict)
    dayoops_cf.insert(day_key, {now_uuid:oopsid})
    return day_key
